/*
Copyright (C) 2009 -  Claudio Salvadori and Paolo Pagano
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/**
 *\file LoopDAQEvent_Sniffer.cpp
 *\date  Created on: 26-08-2009
 *\author  Author: Claudio Salvadori and Paolo Pagano
 *\brief  Post-processing functions */
 
#include "TApplication.h"
#include "TH1D.h"
#include "TGraph.h"
#include <stdlib.h>
#include "Riostream.h"
#include "TROOT.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TNetFile.h"
#include "TRandom.h"
#include "TTree.h"
#include "TBranch.h"
#include "TClonesArray.h"
#include "TStopwatch.h"
#include "DAQEvent.h"
#include "DAQSData.h"

#define NEVENT 5000

DAQEvent * event_;
TFile* f, *g;
TH1F *hp;
TCanvas *canv_;

void init(char * output){
	g = new TFile ( output, "RECREATE");  
}

int main(int argc, char**argv){
#ifdef DAQSNIFFER
	bool first = true;
	UInt_t before = 0;
	UInt_t delta = 0;
	if ( argc < 3 ) {
		cout << "Synapsis: LoopDAQEvent <in.root> <out.root> " <<  endl;
		exit(1);
	}
	init ( argv[2] );
	
	f= new TFile(argv[1], "UPDATE");
	
	TApplication PostApp("PostApp", &argc, argv);
	hp = new TH1F("Bc","Becon coerency",100, 17, 19);
	
	TAxis *x, *y;
	x = hp->GetXaxis(); 
	y = hp->GetYaxis();
	
	x->SetName("ms");
	y->SetName("Entries");
	
	canv_ = new TCanvas("Beacon_coerency", "Beacon coerency", 400, 400);
	canv_->SetFillColor(0);
	canv_->SetBorderMode(0);
	canv_->SetBorderSize(2);
	canv_->SetFrameBorderMode(0);
	canv_->SetHighLightColor(0);
	canv_->Draw();
	canv_->Update();
	
	hp->Draw();
	
	//   Read Tree named "T" in memory. Tree pointer is assigned the same name
	TTree *T = (TTree*)f->Get("T");

	DAQSData *mydaq;
	DAQPayload *mypayload;
	TBranch *branch  = T->GetBranch("event");
	branch->SetAddress(&event_);
	Int_t nevent = T->GetEntries();
	Int_t nselected = 0;
	
	for (Int_t i=1;i<nevent;i++) {

		if(i%50 == 0) printf("DAQEvent:%d\n",i);

		T->GetEntry(i);       
		 
		if (event_->GetNDAQSDatas() > 687)continue;

		TClonesArray *myarray = event_->GetDAQSDatas();
		if (myarray) mydaq = (DAQSData *) myarray->First();
		if (!mydaq) continue;
		cout << "Pointer is: " << myarray << endl;

		mydaq->printData();


		mypayload = mydaq->getPayload();
		cout << "Timestamp: " << mypayload->timestamp_ << endl; 
		
		if(first){
			before = mypayload->timestamp_;
			first = false;
		}else{
			delta = mypayload->timestamp_ - before;
			before = mypayload->timestamp_;
			cout << "Delta: " << delta << endl;
			hp->Fill((float)delta / 1000);
			hp->Draw();
			canv_->Modified();
			canv_->Update();
			
		}
		nselected++;
		event_->Clear();     //clear tracks array
	}
	hp->Write();	
	getchar();
	
	f->Close();
	g->cd();
	g->Write();
#else
	cout << "Error: this executable has not been compiled with the DAQSOCKET symbol." <<  endl;
	cout << "Recopile with: make socket" <<  endl;	
	exit(1);
#endif  
}
