/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _DAQ_ENUM_H_
#define _DAQ_ENUM_H_

/**
 *\file DAQ_enum.h
*
 *\date  Created on: 1-mag-2009
 *\author  Author: Claudio Salvadori
 *\brief  Enumerations for the UI CDaq and data types definition
 */

/** \brief Definition of type size in bytes (unsigned types have the same size of signed ones)*/
#define INT8_S 		1	/**< T_INT8 and T_UINT8 size in number of byte*/
#define INT16_S		2	/**< T_INT16 and T_UINT16 size in number of byte*/
#define INT32_S		4	/**< T_INT32 and T_UINT32 size in number of byte*/
#define INT64_S 	8	/**< T_INT64 and T_UINT64 size in number of byte*/
#define FLOAT32_S 	4	/**< T_FLOAT32 size in number of byte*/
#define FLOAT64_S 	8	/**< T_FLOAT64 size in number of byte*/

/**
*\brief Enumeration of available decoder for packet
*/
enum decoder_type{
	D_NONE			=	0,	/**< No decoder*/
	D_XOR			=	1,	/**< Decoder Xor*/
	/********************************************************/
	/*                  ADD HERE THE NEW DECODER            */
	/********************************************************/	
};

/**
*\brief Enumeration of possible data type defined in the xml file
*/
enum var_type{
	T_NOTYPE		=	0,	/**< Error: not correct type*/
	T_INT8			=	1,	/**< Integer type (8bit)*/
	T_UINT8			=	2,	/**< Integer type (8bit) unsigned*/
	T_INT16			=	3,	/**< Integer type (16bit)*/
	T_UINT16		=	4,	/**< Integer type (16bit) unsigned*/
	T_INT32			=	5,	/**< Integer type (32bit)*/
	T_UINT32		=	6,	/**< Integer type (32bit) unsigned*/
	T_INT64			=	7,	/**< Integer type (64bit)*/
	T_UINT64		=	8,	/**< Integer type (64bit) unsigned*/
	T_FLOAT32		=	9,	/**< Float type (32bit)*/
	T_FLOAT64		=	10,	/**< Float type (64bit)*/	
};

/**
*\brief Enumeration of possible stream type defined in the xml file
*/
enum stream_type{
	T_SERIAL		=	0,	/**< Stream from RS-232*/
	T_USB			=	1,	/**< Stream from USB*/
	T_FILE			=	2,	/**< File parsing*/
	T_STDIN			=	3,	/**< Stdin stream*/
	T_PIPE			=	4,	/**< Steam from a thread pipe*/
	T_SOCKET		=	5,	/**< Steam from a internet socket*/
	/**************************************************************************/
	/*                  ADD HERE THE NEW STREAM TYPE                          */
	/**************************************************************************/	
};

/**
*\brief Enumeration of possible endianess defined in the xml file
*/
enum endianess_type{
	E_BIG			=	0,	/**< Big endian*/
	E_LITTLE		=	1,	/**< Little endian*/
};

/**
*\brief Enumeration of possible type of handshake in the xml file
*/
enum handshake_style{
	H_S_STANDARD		=	0,	/**< Standard handshake*/
	H_S_NONE		=	1,	/**< No handshake*/
	H_S_START		=	2,	/**< Handshake only for starting communication*/
};

#endif
