/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef IRENDERER_H
#define IRENDERER_H

#include <string>
#include <map>
#include <vector>

#include "DAQ_enum.h"
/**
 *\file IRenderer.h
 *\date  Created on: 05-oct-2009
 *\author  Author: Claudio Salvadori
 *\brief   Interface to render data (images, histograms, graphs, tables)*/

/**
*\class IRenderer
*\brief Interface to render data (images, histograms, graphs, tables)
*/

class IRenderer{ 
          public: 
          
          	virtual ~IRenderer() {};
				
		/**\brief This method render data
		*\param data data to be rendered (input)
		*/
		virtual void render(void* data, unsigned long size) = 0;

		/**
		*\class Factory
		*\brief IRenderer object factory to handle the renderer 
		*/
		class Factory {
		public:
			/**\brief Contructor: permit to link a string with a method to create a certain renderer
			* 	using maps.
			* \param semantic	name of the renderer
			* \param creator	method to create the respective renderer
			*/				
			Factory(std::string semantic, IRenderer *(*creator)(const void* node));
			
			/**\brief IRenderer has to access to Factory privete and static elements*/
			friend class IRenderer;
		private:
			static std::map<std::string, IRenderer *(*)(const void*)> 	*creators; 	/**<Renderer creator map*/
			static std::vector<IRenderer*> 					*renderers;	/**<Renderer object vector*/
			static std::vector<std::string>					*nameV_;	/**<Renderer name vector*/
		};
		
		/**\brief This method set the data type inside the renderer object
		* \param vType	data type to set
		*/
		virtual void setType(var_type vType) = 0;
		
		/**\brief This method create the renderer, add it to the renderer vector and fix data type in it
		* \param semantic	renderer name
		* \param node		generic node that contains the renderer presets
		* \param vType		data type to set
		*/
		static void create(std::string semantic, const void* node, var_type vType);

		/**\brief This method return the pointer of the i-th renderer
		* \param i	renderer position
		* \return The pointer to the renderer
		*/
		static IRenderer *getRenderer(unsigned int i);

		/**\brief This method check the correctness of the object name
		* \param name	object name to check
		*/
		static void checkName(std::string name);
		
		/**\brief This method delete the name vector*/
		static void endCheck(void);
		
};
#endif
