/*
Copyright (C) 2009 -  Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file CFile.cpp
 *\brief  File reading functions development
 */

#include <stdio.h>

#include "CFile.h"

using namespace std;

CFile::~CFile(){
	fclose(fdn);
}

s_ret CFile::File_handshaking(handshake_style style){
	return S_RET_OK;
}
s_ret CFile::getDefinition (struct SDef def, s_read_style sty){
	f_Name = (char *) def.s_Name.c_str();		
	f_Style = sty;
	
	switch(f_Style){
		case S_STYLE_LENGTH:
		f_Size = def.s_Size;
		#ifdef DEBUG
		printf("Lenght style\n");
		#endif
		break;
	
		case S_STYLE_S_E:
		f_Size = def.s_Size;
		f_Start = (unsigned char)def.s_Option->o_Start;
		f_End = (unsigned char)def.s_Option->o_End;
		#ifdef DEBUG
		printf("Start/end style %c, %c\n", f_Start, f_End);
		#endif
		break;
	}
	
	fdn = fopen(f_Name, "rb");
#ifdef DEBUG
	printf("File name: %s\n", f_Name);
#endif	

	if (fdn == NULL){
		return S_F_OPEN_ERR;
	}
	
	
	/**\todo endianess in header reading*/
	s_ret dummy = S_RET_OK;
	if(!def.s_File){
		return S_RET_OK;
	}else{
		//Reading the header...
		unsigned char* h_buffer = new unsigned char[def.s_File->f_Header];
		if(def.s_File->f_Header){
			dummy = File_read(h_buffer, def.s_File->f_Header);
			if(dummy != S_RET_OK)
				return dummy;
		
			//... extracting the number of events...
			switch(def.s_File->f_Entry_L){
				case 1:
					entries_ = (unsigned long)((unsigned char)h_buffer[def.s_File->f_Entry_S]);
					#ifdef DEBUG
						std::cerr << "Number of entries: " << entries_ << endl;
					#endif
				break;
			
				case 2:
					entries_ = (unsigned long)*((unsigned short*)(&h_buffer[def.s_File->f_Entry_S]));
					#ifdef DEBUG
						std::cerr << "Number of entries: " << entries_ << endl;
					#endif
				break;
			
				case 4:
					entries_ = (unsigned long)*((unsigned int*)(&h_buffer[def.s_File->f_Entry_S]));
					#ifdef DEBUG
						std::cerr << "Number of entries: " << entries_ << endl;
					#endif
				break;
			
				case 8:
					entries_ = *((unsigned long*)(&h_buffer[def.s_File->f_Entry_S]));
					#ifdef DEBUG
						std::cerr << "Number of entries: " << entries_ << endl;
					#endif
				break;
			
				default:
					entries_ = 0;
					#ifdef DEBUG
						std::cerr << "No entries!"<< endl;
					#endif
				break;
			}
		}else{
			entries_ = 0;
		}
	}
	return S_RET_OK;	
}


s_ret CFile::retStream(unsigned char **buffer, unsigned long *length){
	/**\todo Introduce other feof check and ferror controls*/
	unsigned long dim;
	switch(f_Style){
		case S_STYLE_LENGTH:
		*buffer = new unsigned char[f_Size];
		dim = fread(*buffer,  1, f_Size, fdn);
		#ifdef DEBUG
		printf("Buffer length: %u\n", dim);
		for(unsigned int i = 0; i < dim; i++){
			printf("Buffer read: %x", **buffer + i);
		}
		printf("\n");
		#endif
		if (dim >= 0){
			if(dim == f_Size){
				*length = dim;
				return S_RET_OK;
			} else {
				if(dim == 1){
					*length = 0;
					memset (*buffer, 0, f_Size);
					return S_F_EOS;
				}else{
					*length = dim;
					memset (*buffer, 0, f_Size);
					return S_F_R_LENGTH_ERR;	
				}	
			}
		}else 
			return S_F_R_LENGTH_ERR;
		break;
		
		case S_STYLE_S_E:
		std::vector<unsigned char> buff;
		unsigned char buff_t;
		
		//Searching for syncro character (start character)...
		while(buff_t != f_Start){
			//check the size of the characters read
			if (dim < 0) 
				return S_F_R_LENGTH_ERR;
			if (feof(fdn) != 0)
				return S_F_EOS;
				
			dim = fread(&buff_t, 1, 1, fdn);
		}
		
		//clear the buffer
		buff.clear();

		//... and reading untill end character
		while(1){ 
			dim = fread(&buff_t, 1, 1, fdn);
			
			//check the size of the characters read
			if (dim < 0) 
				return S_F_R_LENGTH_ERR;
			if (dim == 0)
				return S_F_EOS;
			//End character found	
			if((buff_t == f_End) && (buff.size() >= f_Size)){
				*length = buff.size();
				*buffer = new unsigned char[*length];
				
				for(unsigned long i_s = 0; i_s < *length; i_s++) 
					(*buffer)[i_s] = buff[i_s];
				
				#ifdef DEBUG
				for(unsigned long i_s = 0; i_s < *length; i_s++){
					printf("Buffer: %d\n", (*buffer)[i_s]);
				}
				#endif
				return S_RET_OK;
			}
			
			//push back the character
			buff.push_back(buff_t);
		};
		break;
	}
	return S_RET_OK;
}

s_ret CFile::File_read(unsigned char *data, unsigned int data_size){
	unsigned long dim = fread(data,  1, data_size, fdn);
	if(dim < 0)
		return S_F_R_LENGTH_ERR;
	return S_RET_OK;
}

s_ret CFile::File_write(unsigned char* data, unsigned int data_size){
	return S_RET_OK;
}


